﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Optimization;
using System.Threading;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// Can execute an algorithm synchronously
  /// </summary>
  public class AlgorithmExecutor {
    private EngineAlgorithm algorithm;
    private AutoResetEvent waitHandle = new AutoResetEvent(false);

    public AlgorithmExecutor(EngineAlgorithm algorithm) {
      this.algorithm = algorithm;
    }

    public void StartSync() {
      algorithm.Stopped += new EventHandler(algorithm_Stopped);
      algorithm.Paused += new EventHandler(algorithm_Paused);
      algorithm.Start();
      waitHandle.WaitOne();
      waitHandle.Dispose();
      algorithm.Stopped -= new EventHandler(algorithm_Stopped);
      algorithm.Paused -= new EventHandler(algorithm_Paused);
    }

    void algorithm_Paused(object sender, EventArgs e) {
      waitHandle.Set();
    }

    void algorithm_Stopped(object sender, EventArgs e) {
      waitHandle.Set();
    }
  }
}
